/* NIGHTFALL Light Curve Synthesis Program                                 */
/* Copyright (C) 1998 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */


#include <math.h>
#include <stdio.h>
#include <string.h>
#include <float.h>
#include "Light.h"

/* 2002-05-03 rwichmann HIGH_PRECISION code
 */
#ifdef HIGH_PRECISION

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.2
 @short     Numerically differentiate dR/deta
 @param     (int)  Comp    The stellar component
 @return    (double)       dR/deta
 @heading   Star Setup
 ****************************************************************************/
double NumDiffR (double deltaIn, double etaIn, double sinphi,
                 double RadiusBound, double RXCrit, int * testerr)
{
  double val1, val0;
  double eps = pow(DBL_EPSILON, (1.0/3.0));
  double delta;
  double nu_store;

  delta = eps * deltaIn;
  delta = (delta < eps) ? eps : delta;

  nu_store = nu;

  nu    = sinphi * sin(etaIn-delta);
  val0  = RootFind(RocheSurface, RadiusBound, RXCrit,
                   100.0 * DBL_EPSILON,
                   "NumDiffR", testerr);
  if (*testerr == 1)
    return(0.0);

  nu    = sinphi * sin(etaIn+delta);
  val1  = RootFind(RocheSurface, RadiusBound, RXCrit,
                   100.0 * DBL_EPSILON,
                   "NumDiffR", testerr);
  if (*testerr == 1)
    return(0.0);

  nu = nu_store;
  return ((val1-val0)/(2.0*delta));
}


/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.2
 @short     Numerically differentiate dR/dphi
 @param     (int)  Comp    The stellar component
 @return    (double)       dR/dphi
 @heading   Star Setup
 ****************************************************************************/
double NumDiffRPhi (double deltaIn, double phiIn, double sineta,
                    double RadiusBound, double RXCrit, int * testerr)
{
  double val1, val0;
  double eps = pow(DBL_EPSILON, (1.0/3.0));
  double delta;
  double nu_store;

  delta = eps * deltaIn;
  delta = (delta < eps) ? eps : delta;

  nu_store = nu;

  nu    = sineta * sin(phiIn-delta);
  val0  = RootFind(RocheSurface, RadiusBound, RXCrit,
                   100.0 * DBL_EPSILON,
                   "NumDiffRPhi", testerr);
  if (*testerr == 1)
    return(0.0);

  nu    = sineta * sin(phiIn+delta);
  val1  = RootFind(RocheSurface, RadiusBound, RXCrit,
                   100.0 * DBL_EPSILON,
                   "NumDiffRPhi", testerr);
  if (*testerr == 1)
    return(0.0);

  nu = nu_store;
  return ((val1-val0)/(2.0*delta));
}
/* #ifdef HIGH_PRECISION
 */
#endif


/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.2
 @short     Divide stellar surface into grid
 @long      First, set up the grid, then compute the
            temperature distribution on the grid, including
            spots and reflection
 @param     (int)  Comp    The stellar component
 @return    (int)          The exit status
 @heading   Star Setup
 ****************************************************************************/
int DivideSurface(int Comp)
{
  register long i,j;                  /* loop variables                  */
  long n_phi;                         /* # of surface steps in phi       */
  long n_eta;                         /* # of surface steps in eta       */
  long NumElem;                       /* current surface element         */
  long NextPtr;                       /* next    surface element         */
  int  testerr = 0;                   /* exit status of RootFind         */
  
  double     eta;                     /* eta of surface element           */
  double     phi;                     /* phi of surface element           */
  double     sineta, sinphi;          /* sin,cos of phi                   */
  double     coseta, cosphi;          /* sin,cos of eta                   */
  double     rad1;                    /* radius of surface element        */
  double     sqr_rad1;                /* square of radius                 */
  double     delta_eta;               /* step in eta                      */
  double     delta_phi;               /* step in phi                      */
  double     rho;                     /* rho of surface element           */
  /* 2002-05-03 rwichmann HIGH_PRECISION code
   */
#ifdef HIGH_PRECISION
  double     rdiff;                   /* dR/deta                          */
  double     rdiff2;                  /* dR/dphi                          */
#endif
  double     cub_rho;                 /* cube of rho                      */
  double     C_eta, C_phi, C_rad1;    /* surface gravity vector           */
                                      /* in spherical coordinates         */
  double     C_x, C_y, C_z;           /* surface gravity vector           */
                                      /* in cartesian coordinates         */
  double     grav;                    /* surface gravity                  */
  double     lvec, mvec, nvec;        /* surface normal vector            */
  double     RXCrit;                  /* Critical radius                  */
  double     DSurf;                   /* surface of element               */
  double     MeanGrav = 0.0;          /* mean gravity                     */
  double     Surf = 0.0;              /* total surface                    */
  double     MeanDarkGrav = 0.0;      /* contribution to grav. darkening  */
  double     RadiusBound = 0.0;       /* inner bound for RootFind         */
  double     T1;                      /* temperature of this star         */
  double     T2;                      /* temperature of the other star    */
  double     Tr;                      /* temperature ratio^4              */
  double     etacorr_over, eta_corr, deltaeta_corr; /* Throat adjustment  */
  double     eta_next;                              /* Pointer setting    */
  long       n_next, n_last, n_comp;                /* Pointer setting    */
  /*  for mean temperature calculation                                    */
  double     Area;                    /* area of element                  */
  double     Weight=0.0;              /* total area                       */
  double     TempMean=0.0;            /* mean temperature                 */

  SurfaceElement  *SurfPtr;           /* pointer to surface               */
  BinaryComponent *BinPtr;            /* pointer to binary                */

  
  /* ---------- initialize ---------------------------------------------- */

  SurfPtr      = Surface[Comp];
  BinPtr       = &Binary[Comp];

  Mq           = Binary[Comp].Mq;
  if      (Comp == Primary && Flags.asynchron1 == ON)
    F            = Binary[Comp].Fratio;
  else if (Comp == Secondary && Flags.asynchron2 == ON)
    F            = Binary[Comp].Fratio;
  else
    F            = 1.0;

  RochePot     = Binary[Comp].RochePot; 
  RadiusBound  = 0.8*Binary[Comp].Radius;
  etacorr_over = Binary[Comp].LimEta; 


  /* ---------- Maximum Radius to search for Root ----------------        */

  if (Flags.fill == OFF) {
    RXCrit = BinPtr->RXCrit;
  } else {
    RXCrit = BinPtr->LimRad  
      + (0.07*SQR(Binary[Primary].Mq))
      +  0.02*Binary[Primary].Mq;
    
    if ( (Binary[Primary].Mq - 1.0) >= FLT_EPSILON) 
      RXCrit = BinPtr->LimRad  
	+ (0.07*SQR(Binary[Secondary].Mq))
	+  0.02*Binary[Secondary].Mq;
    
    if (   (Binary[Primary].RocheFill - 1.05) <= FLT_EPSILON 
	   && (Binary[Primary].Mq - 0.2) >=  FLT_EPSILON
	   && (Binary[Primary].Mq - 5.0)  <= FLT_EPSILON   ) 
      RXCrit = BinPtr->RXCrit + Binary[Primary].RocheFill - 1.;
    
    else if ( (Binary[Primary].RocheFill - 1.02)  <= FLT_EPSILON )
      RXCrit = BinPtr->RXCrit + Binary[Primary].RocheFill - 1.;

				           
    /* critical angle w/r to z                                      */
    etacorr_over = BinPtr->RZatL1 / 
      sqrt(   BinPtr->RZatL1 * BinPtr->RZatL1 
	      + BinPtr->RLag1 *  BinPtr->RLag1);
    etacorr_over = asin( CLAMP(etacorr_over, -1.0, 1.0) );
  }


  BinPtr->Surface = 0.0; 
  BinPtr->Gravity = 0.0;

  if (Comp == 0) { 
        T1 =  BinPtr->Temperature;
        T2 =  Binary[Comp+1].Temperature;
  } else {
        T1 =  BinPtr->Temperature;
        T2 =  Binary[Comp-1].Temperature;
  }
  Tr = T2/T1;
  Tr = Tr * Tr * Tr * Tr;
  n_eta = StepsPerPi; 
  n_phi = 0;

  if (Flags.fill == OFF) BinPtr->LimEta = 0.0;

  delta_eta = (PI - BinPtr->LimEta)/n_eta; 
  eta = PI-(0.5*delta_eta);   /* start value, point opposite to companion */ 
  
  if (Flags.fill == ON) {    /* Throat adjustment                         */
     etacorr_over = (BinPtr->LimEta - etacorr_over) 
                     + 3.0 * delta_eta;
  }

  NumElem    = 0;

  /* -------------------  loop over eta  ------------------------------   */

  for (i = 0; i < n_eta; ++i) {    

    eta_corr      = eta;         /* Throat adjustment */
    deltaeta_corr = delta_eta;

    phi        = 0.0;
    sineta     = sin(eta); 
    coseta     = cos(eta);
    lambda     = coseta;


    /* fix n_phi for GL animation, because the vertex list needs a        */
    /* fixed number vertices per phi loop                                 */
#ifdef _WITH_OPENGL
    if (Flags.animate == OFF) {
#endif
      /* n_phi proportional to sin(eta)                                   */
      if ( fabs(2 * n_eta * sineta) >= 0.0 )
	n_phi = (long) (10 + 1 + (int)(fabs(2 * n_eta * sineta)));
      else n_phi = 10;
#ifdef _WITH_OPENGL
    } else {
      /* n_phi fixed to a power of 2 for GL animation                     */
      n_phi = 32;
    }
#endif

    delta_phi = (PI+PI)/n_phi; 

    /* check if enough memory                                             */
    if ((NumElem+n_phi) >= (int) MaximumElements) {
#ifdef _WITH_GTK
      if (Flags.interactive == OFF)
	nf_error(_(errmsg[10]));
      else {
	make_dialog(_(errmsg[10]));
        return(8);
      }
#else
      nf_error(_(errmsg[10]));
#endif
    }
    
    BinPtr->N_PhiStep[i] = n_phi;  /* store number of phi intervals */

    /* ---------  Loop over phi ---------------------------------------- */

    for (j = 0; j < n_phi; ++j) {    

       sinphi     = sin(phi); 
       cosphi     = cos(phi);

      /* Throat adjustment for last three rings                          */
      if ((Flags.fill == ON) && (i > (n_eta-4))) {  
        if (i == n_eta-3) eta_corr = eta - ((etacorr_over/6.0)
                                            -(delta_eta/2.0))*fabs(sinphi);
        else if (i == n_eta-2) eta_corr = eta - ((etacorr_over/2.0) 
                                            -(1.5*delta_eta))*fabs(sinphi);
        else if (i == n_eta-1) eta_corr = eta - ((5.0*etacorr_over/6.0) 
					    -(5.0*delta_eta/2.0))*fabs(sinphi);

        sineta        = sin(eta_corr); 
        coseta        = cos(eta_corr);
        lambda        = coseta; 
        deltaeta_corr = delta_eta+((etacorr_over/3.0)-delta_eta)*fabs(sinphi);
      }       

       mu         = sineta*cosphi;
       nu         = sineta*sinphi;
 
       rad1       = RootFind(RocheSurface, RadiusBound, RXCrit, 
			     100.0 * DBL_EPSILON, /* was 1.0e-8, */ 
			     "DivideSurface1", &testerr);
       if (testerr == 1) return(8);

       sqr_rad1   = rad1*rad1;
       rho        = 1.0/sqrt(1.0+(sqr_rad1)-2.0*rad1*lambda);
       cub_rho    = rho*rho*rho;

       /* old code 
       C_rad1     = -1.0/(sqr_rad1) + Mq*(cub_rho*(lambda-rad1)-lambda)
                      +F*F*(Mq+1.0)*(sqr_rad1)*(1.0- nu*nu);
       */
       /* 2002-05-03 rwichmann error in expression fixed 
	*/
       C_rad1     = -1.0/(sqr_rad1) + Mq*(cub_rho*(lambda-rad1)-lambda)
                      +F*F*(Mq+1.0)*rad1*(1.0- nu*nu);
       C_eta      = sineta
                      *(Mq*(1.0-cub_rho)-(Mq+1.0)*rad1*F*F
			*lambda*sinphi*sinphi);
       C_phi      = -(Mq + 1.0)*F*F*rad1*nu*cosphi;
       C_x        = C_rad1*lambda - C_eta*sineta;
       C_y        = C_rad1*mu + C_eta*lambda*cosphi - C_phi*sinphi;
       C_z        = C_rad1*nu + C_eta*lambda*sinphi + C_phi*cosphi;

       grav       = sqrt( C_x*C_x + C_y*C_y + C_z*C_z );

       lvec       = -C_x/grav;
       mvec       = -C_y/grav;
       nvec       = -C_z/grav;

       DSurf      = (sqr_rad1) * sineta * deltaeta_corr * delta_phi;

       /* 2002-05-03 rwichmann HIGH_PRECISION code */

#ifdef HIGH_PRECISION
       /*
        * dA = sin(eta) deta dphi * ( R*R e(r) - R dR/deta e(eta)
        *                             - R/sin(eta) dR/dphi e(phi) )
        */

       /* the dR/deta term
        */
       rdiff = NumDiffR (deltaeta_corr, eta_corr, sinphi,
                         RadiusBound, RXCrit, &testerr);
       if (testerr == 1) return (8);
       rdiff = rad1 * rdiff * sineta * deltaeta_corr * delta_phi;

       /* the dR/dphi term
        */
       rdiff2 = NumDiffRPhi (delta_phi, phi, sineta,
                         RadiusBound, RXCrit, &testerr);
       if (testerr == 1) return (8);
       rdiff2 = rad1 * rdiff2 * deltaeta_corr * delta_phi;

       /* absolute value
        */
       DSurf      = sqrt(DSurf*DSurf + rdiff*rdiff + rdiff2*rdiff2);
#else
       /*
        * cos (e) = lambda*lvec+mu*mvec+nu*nvec
        *         = angle to surface normal
        */
       DSurf      = DSurf/(lambda*lvec+mu*mvec+nu*nvec);
#endif

       SurfPtr->eta      = (float) eta_corr;
       SurfPtr->phi      = (float) phi;
       SurfPtr->rad      = (float) rad1;
       SurfPtr->lambda   = (float) (lambda * rad1);
       SurfPtr->mu       = (float) (mu * rad1);
       SurfPtr->nu       = (float) (nu * rad1);

       SurfPtr->rho    = (float) rho;
       SurfPtr->grav   = (float) grav;
       SurfPtr->l      = (float) lvec;
       SurfPtr->m      = (float) mvec;
       SurfPtr->n      = (float) nvec;


       SurfPtr->area   = (float) DSurf;

       SurfPtr->MyNum = NumElem;

       Surf         = Surf + DSurf;
       MeanGrav     = MeanGrav + (grav*DSurf);
       MeanDarkGrav = MeanDarkGrav 
                     + exp(BinPtr->GravDarkCoeff * log(grav))*DSurf;


       /*  ---------------    INITIALIZE SURFACE TEMP  ----------------- */

       SurfPtr->temp = T1;

       phi        = phi + delta_phi;  /* increment phi                   */
       ++NumElem;                     /* increment surface element count */
       ++SurfPtr;

    }  /* -----------  end loop over phi ---------------------------     */

    eta = eta - delta_eta;

  }  /* -------------- end loop over eta ------------------------------- */


  BinPtr->NumElem = NumElem;
  BinPtr->NumPlus = NumElem;

  if (Flags.debug[VERBOSE] == ON) { 
    printf("\n"); 
    printf(_(" Divide Star %3d : %6ld Surface elements\n"),
	   Comp+1, BinPtr->NumElem);
  }

  BinPtr->Surface       = Surf;
  BinPtr->Gravity       = MeanGrav/Surf;
  BinPtr->DarkeningGrav = MeanDarkGrav/Surf;


  /* ------------------- SET next/last_ptr ---------------------------- */

  /* we need this only once                                             */

  if (Flags.first_pass == ON) { 
 
  SurfPtr      = Surface[Comp];

  n_last     = 0;
  n_next     = 0;
  NumElem    = 0;

  for (i = 0; i < n_eta; ++i) {  

    eta_next    = eta - delta_eta;                   /* pointer setting */

    if (i > 0) n_last = n_phi;                       /* pointer setting */
    n_phi = BinPtr->N_PhiStep[i];
    if ( i < n_eta-1) n_next = BinPtr->N_PhiStep[i+1];

   for (j = 0; j < n_phi; ++j) {

      
     /* -----------     the first ring   -----------------------------  */
     if (i == 0) { 
	       SurfPtr->last_ptr = NULL; 
               if (n_next == n_phi) {
                         SurfPtr->next_ptr = 
                                    &Surface[Comp][NumElem+n_next];
	       } else {
                 n_comp = ROUND(((float)(n_next)/(float)(n_phi))*(float)(j));
                 n_comp = n_phi - j + n_comp;
                         SurfPtr->next_ptr = 
                                    &Surface[Comp][NumElem+n_comp]; 
	       }
     }
     /* -----------     somewhere in the middle  ---------------------  */
     else if ( (i > 0) && (i < (n_eta-1))) {
               if (n_next == n_phi) {
                 NextPtr = NumElem+n_next;
                 if (NextPtr >= BinPtr->NumElem)  
                       NextPtr = BinPtr->NumElem-1;
                         SurfPtr->next_ptr = 
                                    &Surface[Comp][NextPtr];
	       } else {
                 n_comp = ROUND(((float)(n_next)/(float)(n_phi))*(float)(j));
                 n_comp = n_phi - j + n_comp;
                 NextPtr = NumElem+n_comp;
                 if (NextPtr >= BinPtr->NumElem) 
                       NextPtr = BinPtr->NumElem-1;
                         SurfPtr->next_ptr = 
                                    &Surface[Comp][NextPtr]; 
	       }
              if (n_last == n_phi) {
                 SurfPtr->last_ptr = 
                                    &Surface[Comp][NumElem-n_next];
	       } else {
                 n_comp = ROUND(((float)(n_last)/(float)(n_phi))*(float)(j));
                 n_comp = (n_last - n_comp) + j; 
                         SurfPtr->last_ptr = 
                                    &Surface[Comp][NumElem-n_comp]; 
	       }
     }

     /* -----------     the last  ring   -----------------------------  */

     /*@ignore@*/
     /* Storage SurfPtr->last_ptr is kept in one path, but live in      */
     /*    another.                                                     */

     else  if (i == (n_eta-1)) {

       if (n_last == n_phi) { 
	 NextPtr = NumElem-n_phi;
	 if (NextPtr >= BinPtr->NumElem) 
	   NextPtr = BinPtr->NumElem-1;
	 SurfPtr->last_ptr =  
	   &Surface[Comp][NextPtr];
       } else { 
	 n_comp = ROUND(((float)(n_last)/(float)(n_phi))*(float)(j));
	 n_comp = (n_last - n_comp) + j;  
	 NextPtr = NumElem-n_comp -1;
	 if (NextPtr >= BinPtr->NumElem) 
	   NextPtr = BinPtr->NumElem-1;
	 SurfPtr->last_ptr =  
	   &Surface[Comp][NextPtr];  
       }

       if (Flags.fill == OFF) {
	 n_comp = (j + (n_phi/2)) % n_phi;
	 n_comp = - j + n_comp;
	 NextPtr = NumElem+n_comp;
	 SurfPtr->next_ptr = 
	   &Surface[Comp][NextPtr];
       } else {
	 if (Comp == Primary) {
	   SurfPtr->next_ptr = 
	     &Surface[Secondary][NextPtr];
	 } else {
	   NextPtr = NumElem;
	   SurfPtr->next_ptr = 
	     &Surface[Primary][NextPtr];
	 }
       }
     }
     

     if (NumElem != 0) {
       if (j != 0) {
	 SurfPtr->phi_ptr = 
	   &Surface[Comp][NumElem-1];
       } else {
	 SurfPtr->phi_ptr =
	   &Surface[Comp][NumElem+n_phi-1];
       }
       if (j != (n_phi-1)) {
	 SurfPtr->phi1_ptr = 
	   &Surface[Comp][NumElem+1];
       } else {
	 SurfPtr->phi1_ptr =
	   &Surface[Comp][NumElem-(n_phi-1)];
       }
     } else {
       SurfPtr->phi1_ptr = 
	 &Surface[Comp][1];
       SurfPtr->phi_ptr = 
	 &Surface[Comp][1];
     }
     
     ++NumElem;
     ++SurfPtr;
     
   }
  /*@end@*/
  }

  }

  /* -------------- END SET next/last_ptr ----------------------------- */


  /**********************************************************************/
  /*                                                                    */
  /*               Temperature Distribution                             */
  /*                                                                    */
  /**********************************************************************/

  /* --------------- Gravitational Darkening  ------------------------- */

  /* T propto g**GravDarkCoeff                                          */

  SurfPtr      = Surface[Comp]; 

  for (i = 0; i < BinPtr->NumElem; ++i) { 

    SurfPtr->temp =  
      BinPtr->Temperature 
      * (exp(BinPtr->GravDarkCoeff * log(SurfPtr->grav))  
	 / BinPtr->DarkeningGrav );

    ++SurfPtr; 
 
  }

  /* ------------------------   Spots            --------------------   */


  /* make spots or initialize the dimfactor to 1.0                      */
  
  if (Flags.first_pass == ON) {
    if (Comp == Primary) {
      if (Flags.Spots1 > OFF) { 
        MakeSpots(Primary,   0); 
      } else { 
        SurfPtr      = Surface[Primary];  
        for (j = 0; j < Binary[Primary].NumElem; ++j) { 
          SurfPtr->SumDimFactor = 1.0;
          ++SurfPtr;
	}
      }
    }

    if (Comp == Secondary) {
      if (Flags.Spots2 > OFF) { 
	MakeSpots(Secondary, 0);
      } else {
        SurfPtr      = Surface[Secondary];   
	for (j = 0; j < Binary[Secondary].NumElem; ++j) { 
                       SurfPtr->SumDimFactor = 1.0;
		       ++SurfPtr;
	}
      }
    }

  } else {

    /* move spots if asynchroneous rotation or elliptic orbit           */

    if (Comp == Primary) {
      if ( (Flags.Spots1 > OFF) && 
         ( Flags.asynchron1 == ON 
	   || Flags.elliptic == ON) ) 
                   MakeSpots(Primary, Orbit.PhaseIndex);
    }

    if (Comp == Secondary) {
      if ( (Flags.Spots2 > OFF) && 
         ( Flags.asynchron2 == ON 
	   || Flags.elliptic == ON) ) 
                   MakeSpots(Secondary, Orbit.PhaseIndex);
    }

  }

  /* --------------------------------- Reflection  -------------------- */

  if ((Flags.reflect > OFF) && (Comp == Secondary)) {
       LightReflect();
  } 

  if (Flags.reflect == OFF) SimpleReflect(Comp);
  
  /* ---------------------------- mean temperature  -------------------- */

  Weight = 0.0; TempMean = 0.0;
  SurfPtr      = Surface[Comp];
  for (i = 0; i < BinPtr->NumElem; ++i) {
    Area     = SurfPtr->area;
    TempMean = TempMean + SurfPtr->temp * Area;
    Weight   = Weight + Area;
    /*------- Sollte hier nicht irgendwo ein SurfPtr++ stehen?????*/
    /* eigentlich schon ... RW Tue Sep 11 12:54:20 CEST 2001       */
    ++SurfPtr;
  }
  BinPtr->TempMean = TempMean/Weight;
    


  /**********************************************************************/
  /*                                                                    */
  /*               Flux intensity computation                           */
  /*                                                                    */
  /**********************************************************************/

  if (Comp == Secondary) {
     if (Flags.blackbody == ON) { 
        BlackbodyFlux(Primary);
        BlackbodyFlux(Secondary);
     } else {
        ModelFlux(Primary);
        ModelFlux(Secondary);
     }
  }

  /* ----------------           output            --------------------- */

  if (Flags.debug[VERBOSE] == ON) { 
         /* printf("\n Star: %5d\n", (Comp+1)); */
    printf(_(" Surface Area, Mean Gravity (dimensionless):  %8.6f %8.6f\n"),
	   BinPtr->Surface, BinPtr->Gravity);
    printf(_(" Temperature is %8.6f\n"), BinPtr->Temperature);
    
    printf("\n");
    printf(_(" Temperature Distribution corrected for: \n")); 
    printf(_("    Reflection and Gravity Darkening\n"));
    printf(_("    Coefficient for Gravity Darkening was: %6.4f\n"),
	   BinPtr->GravDarkCoeff);
  }

  return(0);

}

#ifdef HAVE_DISK
/* Can we rename this to a more intuitive name light rotatex or what ever; 
   I don't know what "tile" in this context means; Why using floats and 
   not double values ? MK */ 
void TileDisk1(double TiltAngle,SurfaceElement *SurfPtr)
{
  float lambda_strich;
  float nu_strich;

  lambda_strich   = (float) SurfPtr -> lambda*cos(TiltAngle)-
                            SurfPtr -> nu*sin(TiltAngle);
  nu_strich       = (float) SurfPtr -> lambda*sin(TiltAngle)+
                            SurfPtr -> nu*cos(TiltAngle); 
 
  SurfPtr->lambda = lambda_strich;
  SurfPtr->nu     = nu_strich;
}

void TileDisk2(double TiltAngle,SurfaceElement *SurfPtr)
{
  float l_strich;
  float n_strich;

  l_strich      = (float) SurfPtr -> l*cos(TiltAngle)+
                          SurfPtr -> n*sin(TiltAngle);
  n_strich      = (float) SurfPtr -> l*sin(TiltAngle)-
                          SurfPtr -> n*cos(TiltAngle);

  SurfPtr -> l = l_strich;
  SurfPtr -> n = n_strich;
}

void TileDisk3(double TiltAngle,SurfaceElement *SurfPtr)
{
  float lambda_strich;
  float mu_strich;

  lambda_strich  = (float) SurfPtr -> lambda*cos(TiltAngle)-
                           SurfPtr -> mu*sin(TiltAngle);

  mu_strich      = (float) SurfPtr -> lambda*sin(TiltAngle)-
                           SurfPtr -> mu*cos(TiltAngle);

  SurfPtr -> lambda = lambda_strich;
  SurfPtr -> mu = mu_strich;
}

void CalculateLayerVecE1(SurfaceElement *SurfPtr)
{
  double vec_en[3];                         /* Normalenvektor der Ebene */

  vec_en[0]=SurfPtr->l;
  vec_en[1]=SurfPtr->m;
  vec_en[2]=SurfPtr->n;		 
  
  if (vec_en[0] != 0)
    {
      SurfPtr->VecE1[0]=-((vec_en[1]+vec_en[2])/vec_en[0]);
      SurfPtr->VecE1[1]=1.0;
      SurfPtr->VecE1[2]=1.0;
    }
  else
    {
      if (vec_en[1] != 0)
	{
	  SurfPtr->VecE1[0]=1.0;
	  SurfPtr->VecE1[1]=-((vec_en[0]+vec_en[2])/vec_en[1]);
	  SurfPtr->VecE1[2]=1.0;
	}
      else
	{
	  SurfPtr->VecE1[0]=1.0;
	  SurfPtr->VecE1[1]=1.0;
	  SurfPtr->VecE1[2]=-((vec_en[0]+vec_en[1])/vec_en[2]);
	}
    }
}

void CalculateLayerVecE2(SurfaceElement *SurfPtr)
{
  double vec_en[3];                         /* Normalenvektor der Ebene */
  double vec_e1[3];                         /* Vektor Ebene 1           */

  vec_en[0]=SurfPtr->l;
  vec_en[1]=SurfPtr->m;
  vec_en[2]=SurfPtr->n;  

  vec_e1[0]=SurfPtr->VecE1[0];
  vec_e1[1]=SurfPtr->VecE1[1];
  vec_e1[2]=SurfPtr->VecE1[2];

  SurfPtr->VecE2[0]=vec_en[1]*vec_e1[2]-vec_en[2]*vec_e1[1];
  SurfPtr->VecE2[1]=vec_en[2]*vec_e1[0]-vec_en[0]*vec_e1[2];
  SurfPtr->VecE2[2]=vec_en[0]*vec_e1[1]-vec_en[1]*vec_e1[0];
		   
}

/****************************************************************************
 @author    Patrick Risse
 @short     Divide disk surface into grid
 @long      First, set up the grid, then compute the
            temperature distribution on the grid, including
            spots and reflection
 @param     (int)  Comp    The stellar component
 @param     (int)  index   ???????
 @return    (int)          The exit status
 @heading   Disk Setup
 ****************************************************************************/
int DivideDisk(int Comp,int index)
{
  register long i,j;                  /* loop variables                  */
  double  n_phi=32.0;                 /* surface steps in phi            */
  long    n_eta;                      /* # of surface steps in eta       */
  long    NumElem;                    /* current surface element         */
  /*long NextPtr;*/                   /* next    surface element         */
  /*int  testerr = 0;*/               /* exit status of RootFind         */
  
  double  phi;                        /* phi of surface element           */
  double  RadiusBound = 0.0;          /* inner bound for RootFind         */
  double  T1;                         /* temperature of this star         */
  double  T2;                         /* temperature of the other star    */
  double  R1;                         /* Roche fill  of this star         */
  double  R2;                         /* Roche fill  of the other star    */
  double  Rout,Rin;                   /* Outer/inner Radius of the Disk   */
  double  etacorr_over;               /* Throat adjustment  */
  /*long       n_next, n_last, n_comp;*/            /* Pointer setting    */
  /*  for mean temperature calculation                                    */
  double  Area;                       /* area of element                  */

  double  Weight=0.0;                 /* total area                       */
  double  TempMean=0.0;               /* mean temperature                 */
  double  aux_area;                   /* area of element, needed for      *
                                       * intermediate computation         */
  double  Phase;                      /* Phase of the System. Is needed
				       for recalculating the Disk Position*/

  SurfaceElement  *SurfPtr;           /* pointer to surface               */
  BinaryComponent *BinPtr;            /* pointer to binary                */

  /* ---------------- Variables added for the disk by P.R. ---------------*/
  double  Thick_out;                  /* Thickness of the disk on the 
                                         outside                          */
  double  Thick_in;                   /* Thickness of the disk on the 
                                         inside                           */
  double  d_s1d;                      /* Area of one element on the top/
                                         bottom side of the disk          */
  double  d_s2d_out;                  /* Area                             */
  double  d_s2d_in;
  double  P_0;                          
  double  DSurf;                      /* surface of element               */
  double  Surf = 0.0;                 /* total surface                    */
  double  r_i_old;
  double  r_i_new;
  double  r_j=0;
  double  alpha;             /*Angles in Degree*/
  double  TiltAngle;
  double  offset_in=0;
  double  offset_out=0;
  double  n_r;                        /* Number of Rings on the top/bottom
                                         of the disk                      */
  /*long    TopElem,OutElem,BottomElem,InElem;*/

  int    NumOfTopElements,NumOfOuterElements ;
  int    NumOfBottomElements,NumOfInnerElements;
  int    NumOfTotalElements;

  double warpfactor[42];          /* Inclination of each ring for the warp*/
  float rot[42];
  int    warp_no;                               /* Entry_No of warpfactor */
  float z_rotation;            /* The disk must counterrotate to be fixed */

  z_rotation=(360.0/(2.0*PI))/PhaseSteps;
  /* printf("z_rotation %f\n",z_rotation); PR */
  n_eta=floor(StepsPerPi/1.5);
  n_r=n_eta;

  /*BinPtr->N_PhiStep[i] = n_phi;*/  /* store number of phi intervals */

  /* ---------- initialize ---------------------------------------------- */

  SurfPtr      = Surface[Comp];
  BinPtr       = &Binary[Comp];

  Mq           = Binary[Comp].Mq;
  if      (Comp == Primary && Flags.asynchron1 == ON)
    F = Binary[Comp].Fratio;
  else if (Comp == Secondary && Flags.asynchron2 == ON)
    F = Binary[Comp].Fratio;
  else
    F = 1.0;

  /* RLag1        = Binary[Comp].RLag1;   This Value is not used. Why is it defined? P.R. 19.6.2001*/
  RochePot     = Binary[Comp].RochePot; 
  RadiusBound  = 0.8*Binary[Comp].Radius;
  etacorr_over = Binary[Comp].LimEta; 

  /* */
  Phase = 0.0;//((2*PI/PhaseSteps) * index) ;
  //printf("PHASE:::: %f\n",Phase);
  
  BinPtr->RocheFill = Binary[Secondary].RocheFill;  
  BinPtr->RCrit = Binary[Secondary].RCrit;  
  BinPtr->RLag1 = Binary[Secondary].RLag1; 

  /* RLag1 is not calculated for the Disk so i take it from the
     Secondary Star. There must be a better possipility to define 
     the RLag1 for the Disk. ToDo: Where is RLag1 defined? 
     P.R. 19.06.2001    */

  Rout  = BinPtr->RLag1*BinPtr->Rout;
  Rin   = BinPtr->RLag1*BinPtr->Rin; /*BinPtr->RocheFill; */


  BinPtr->Surface = 0.0; 
  BinPtr->Gravity = 0.0;

  T1 =  BinPtr->Temperature;
  R1 =  BinPtr->Radius;
  T2 =  Binary[Comp-1].Temperature;
  R2 =  Binary[Comp-1].Radius;


  NumElem    = 0;


  BinPtr->Segments = n_phi;
  /* damn is this a hard job ! It's difficult to set the number of   */
  /* phi  steps to n_phi, this finally took 1 year to be done MK     */
  /* BinPtr->N_PhiStep[0] = n_phi;                                   */

  /* This Function is not tested. There are some bugs inside */
  /* I will fix it later MPR 12.09.2001 */
  if (Flags.warp == ON ) {
    /* Create warpfactor */
    printf("Creat standard warp\n");
    for (i=0; i< n_r-1; i++)
      {
	if (i <= 15) warpfactor[i]=(15.0/360)*2*PI;
	if (i > 15)  warpfactor[i]=((15.0 -((i-15)*0.75))/360)*2*PI;
	if (warpfactor[i] <= 0) warpfactor[i]=0.0;
	if (i >= 1)
	  rot[i]=((2.5/360)*2*PI)*i;
      }
  }


  /* ------------------- Start building Disk grid ---------------------  */
  /* --------------------- General Calculations ------------------------ */

  Thick_out        = BinPtr->Thick+(BinPtr->HR*Rout);          /* Thickness of the Disk on the outer side*/
  Thick_in         = BinPtr->Thick+(BinPtr->HR*Rin);           /* Thickness of the Disk on the inner side*/
  d_s1d            = (PI*((Rout*Rout)-(Rin*Rin)))/(n_r*n_phi); /* Area of one element */
  d_s2d_out        = (2.0*PI*Rout*Thick_out)/n_phi;            /* Area of one element on the outer side */ 
  d_s2d_in         = (2.0*PI*Rin*Thick_in)/n_phi;              /* Area of one element on the inner side */ 
  BinPtr->RingsOut = floor((d_s2d_out/d_s1d)+0.5);             /* Number of rings rounded */
  BinPtr->RingsIn  = floor((d_s2d_in/d_s1d)+0.5);              /* Number of rings rounded */

  P_0              = (Rout*Rout-Rin*Rin)/n_eta;
  r_i_old          = Rin;
  alpha            = atan(BinPtr->HR); /* Angle of rise from the middle to the edge of the Accretion Disk*/
  
  TiltAngle        = (BinPtr->Tilt/360)*2*PI;
  //printf("RingsOut: %f \n",BinPtr->RingsOut);
  //printf("RingsIn: %f \n",BinPtr->RingsIn);


  /* ----------------- Calculate number of elements for the Disk------  */
  NumOfTopElements    = n_phi*n_r;
  NumOfOuterElements  = BinPtr->RingsOut*n_phi;
  NumOfBottomElements = n_phi*n_r;
  NumOfInnerElements  = BinPtr->RingsIn*n_phi;
  NumOfTotalElements  = NumOfTopElements+NumOfOuterElements+NumOfBottomElements+NumOfInnerElements;
 
  printf("NumTOT: %d N_PHI: %f \n",NumOfTotalElements,n_phi);

  /* ---------------------- Calculate top of disk --------------------  */
  /* -----------------  loop over rings (n_r)  ----------------------   */
  for (i = 0; i < n_r; i++) {
    phi        = 0.0;   
    
    /* check if enough memory                                             */
    if ((NumElem+n_phi) >= (int) MaximumElements) {
#ifdef _WITH_GTK
      if (Flags.interactive == OFF)
	nf_error(_(errmsg[10]));
      else {
	make_dialog(_(errmsg[10]));
	return(8);
      }
#else
      nf_error(_(errmsg[10]));
#endif
    }

    r_i_new=sqrt((r_i_old*r_i_old)+P_0);  /* radius of next circle */
    r_j=(r_i_new+r_i_old)/2.0;            /* Center of the element */

    /*compute area of element:*/
    aux_area = PI*((r_i_new*r_i_new)-(r_i_old*r_i_old))/n_phi;
    
    /* ----------------- loop over phi -------------------------------- */
    for (j = 0; j < n_phi; ++j) {
      phi=((2.0*PI/n_phi)*j)-Phase; /* MPR 23.05.2002 */

      SurfPtr->RadiusIn  = r_i_old; /* PR added 07.05.02 is needed for eclipsing*/
      SurfPtr->RadiusOut = r_i_new; /* by disk verification */
          
      SurfPtr->lambda      = (float) r_j*cos(phi);
      SurfPtr->mu          = (float) r_j*sin(phi);
      SurfPtr->nu          = (float) (BinPtr->Thick/2.0+(BinPtr->HR*r_j)/2.0);
      
      SurfPtr->l           = (float) (cos(phi))*sin(alpha);
      if (fabs(SurfPtr->l) <= DBL_EPSILON) {
	SurfPtr->l = 0.0;}
      SurfPtr->m           = (float) (sin(phi))*sin(alpha);
      if (fabs(SurfPtr->m) <= DBL_EPSILON) {
	SurfPtr->m = 0.0;}
      SurfPtr->n           = (float) cos(alpha);
      if (fabs(SurfPtr->n) <= DBL_EPSILON) {
	SurfPtr->n = 0.0;}

      if (Flags.warp == ON) {
	TileDisk1(warpfactor[i],SurfPtr);	  	  
 	TileDisk2(warpfactor[i],SurfPtr);	  	  
      }
      
      TileDisk1(TiltAngle,SurfPtr);	        
      TileDisk2(TiltAngle,SurfPtr);	  

      /* Calculate the layer vectors */
      CalculateLayerVecE1(SurfPtr);
      CalculateLayerVecE2(SurfPtr);

      SurfPtr->AreaType  = TOP_SEGMENT;
      SurfPtr->AreaAngle = j*360/n_phi; /* Angle between x-Axis and Middle of the Element */
      SurfPtr->MyNum     = NumElem;
      SurfPtr->temp      = T2;
      SurfPtr->rho       = 1.00; /* ???*/
      SurfPtr->SpotNum   = 0;
      SurfPtr->rad       = r_j;        /* Is needed for eclipse test */
      DSurf              = aux_area;   /* computed above with rings  */
      SurfPtr->area      =  DSurf;
      Surf               = Surf + DSurf;

      /*  ---------------    INITIALIZE SURFACE TEMP  ----------------- */
      SurfPtr->temp = T1;
      
      ++NumElem;                     /* increment surface element count */
      ++SurfPtr;
      
      /* some elements of the structure are not filled at the moment */
    }  /* end of loop over phi */
    r_i_old=r_i_new; /* end of one ring */
  }     /* end of loop over rings (n_r) */
  
  /* ---------------------- Calculate outer side of disk --------------------  */
  
  /* -----------------  loop over rings (n_r)  ----------------------   */
  for (i = 0; i < BinPtr->RingsOut ; ++i) {
    phi        = 0.0;   /*  Correct ?????*/
    
    /* check if enough memory                                             */
    if ((NumElem+n_phi) >= (int) MaximumElements) {
#ifdef _WITH_GTK
      if (Flags.interactive == OFF)
	nf_error(_(errmsg[10]));
      else {
	make_dialog(_(errmsg[10]));
	return(8);
      }
#else
      nf_error(_(errmsg[10]));
#endif
    }
    
    /* ----------------- loop over phi -------------------------------- */
    for (j = 0; j < n_phi; ++j) {
      offset_out=(Thick_out/BinPtr->RingsOut)/2.0;
      phi=((2.0*PI/n_phi)*j)-Phase; /* MPR 23.05.2002 */
      
      SurfPtr->RadiusIn  = r_i_old; /* PR added 07.05.02 is needed for eclipsing*/
      SurfPtr->RadiusOut = r_i_old; /* by disk verification */

      SurfPtr->lambda      = (float) r_i_old*cos(phi);
      SurfPtr->mu          = (float) r_i_old*sin(phi);
      SurfPtr->nu          = (float) (BinPtr->Thick/2.0+(BinPtr->HR*r_j)/
				      2.0-(offset_out+(i*Thick_out/BinPtr->RingsOut)));
      SurfPtr->l           = (float) cos(phi);
      if (fabs(SurfPtr->l) <= DBL_EPSILON){
	SurfPtr->l = 0.0;}
      SurfPtr->m           = (float) sin(phi); // PR -sin -> sin 21.05.02
      if (fabs(SurfPtr->m) <= DBL_EPSILON){
	SurfPtr->m = 0.0;}
      SurfPtr->n           = 0;

      TileDisk1(TiltAngle,SurfPtr);	      
      TileDisk2(TiltAngle,SurfPtr);	  
      
      /* Calculate the layer vectors */
      CalculateLayerVecE1(SurfPtr);
      CalculateLayerVecE2(SurfPtr);
      
      SurfPtr->AreaType = OUT_RECTANGLE;
      SurfPtr->AreaAngle = j*360/n_phi; /* Angle between x-Axis and Middle of the Element */
      SurfPtr->AreaHeight=Thick_out/BinPtr->RingsOut;
      SurfPtr->AreaWidth =(2*PI*Rout)/n_phi;
      SurfPtr->MyNum = NumElem;
      SurfPtr->rad     = r_j;        /* Is needed for eclipse test */

      /* area is outer side divided by rings/segments:*/
      DSurf            = (2.0*PI*Rout*Thick_out)/n_phi/BinPtr->RingsOut;
      SurfPtr->area    = DSurf;
      Surf             = Surf + DSurf;

      /*  ---------------    INITIALIZE SURFACE TEMP  ----------------- */
      SurfPtr->temp = T1;
      ++NumElem;                     /* increment surface element count */
      ++SurfPtr;
      
      /* some elements of the structure are not filled at the moment */
      
    } /* end of loop over phi         */
  }     /* end of loop over rings (n_r) */

  /* ---------------------- Calculate bottom of disk --------------------  */
  
  /* -----------------  loop over rings (n_r)  ----------------------   */
  for (i = 0; i < n_r; ++i) {
    phi = 0.0;  
    
    /* check if enough memory                                             */
    if ((NumElem+n_phi) >= (int) MaximumElements) {
#ifdef _WITH_GTK
      if (Flags.interactive == OFF)
	nf_error(_(errmsg[10]));
      else {
	make_dialog(_(errmsg[10]));
	return(8);
      }
#else
      nf_error(_(errmsg[10]));
#endif
    }
      
    r_i_new=sqrt((r_i_old*r_i_old)-P_0);  /* radius of next circle */
    r_j=(r_i_new+r_i_old)/2.0;            /* Center of the element */
    
    /*compute area of element:*/
    aux_area = PI*((r_i_new*r_i_new)-(r_i_old*r_i_old))/n_phi;

    warp_no=(int) n_r-i;
    
    /* ----------------- loop over phi -------------------------------- */
    for (j = 0; j < n_phi; ++j) {
      phi=((2.0*PI/n_phi)*j)-Phase;  /* MPR 23.05.2002 */

      SurfPtr->RadiusIn  = r_i_new; /* PR added 07.05.02 is needed for eclipsing */
      SurfPtr->RadiusOut = r_i_old; /* by disk verification 
				     BEWARE!!! r_i_new und r_i_old muessen hier vertauscht sein
				    da die Schleife jetzt von Aussen nach Innen laeuft!!!! */

      SurfPtr->lambda      = (float) r_j*cos(phi);
      SurfPtr->mu          = (float) r_j*sin(phi);
      SurfPtr->nu          = (float) -1.0 * ((BinPtr->Thick/2.0+(BinPtr->HR*r_j)/2.0));
      
      SurfPtr->l           = (float) (cos(phi))*sin(alpha);
      if (fabs(SurfPtr->l) <= DBL_EPSILON) {
	SurfPtr->l = 0.0;}
      SurfPtr->m           = (float) (sin(phi))*sin(alpha);
      if (fabs(SurfPtr->m) <= DBL_EPSILON) {
	SurfPtr->m = 0.0;}
      SurfPtr->n           = (float) -1.0 * (cos(alpha));
      if (fabs(SurfPtr->n) <= DBL_EPSILON) {
	SurfPtr->n = 0.0;}

      if (Flags.warp == ON) {
	TileDisk1(warpfactor[warp_no],SurfPtr);
	TileDisk2(warpfactor[warp_no],SurfPtr);	  
      }

      TileDisk1(TiltAngle,SurfPtr);	  
      TileDisk2(TiltAngle,SurfPtr);	  
      
      /* Calculate the layer vectors */
      CalculateLayerVecE1(SurfPtr);
      CalculateLayerVecE2(SurfPtr);

      SurfPtr->AreaType = BOTTOM_SEGMENT;
      SurfPtr->AreaAngle = j*360/n_phi; /* Angle between x-Axis and Middle of the Element */
      SurfPtr->MyNum    = NumElem;
      SurfPtr->rad      = r_j;        /* Is needed for eclipse test */
      DSurf             = aux_area;   /* computed above with rings  */
      SurfPtr->area     = DSurf;
      Surf              = Surf + DSurf;

      /*  ---------------    INITIALIZE SURFACE TEMP  ----------------- */
      SurfPtr->temp = T1;

      ++NumElem;                     /* increment surface element count */
      ++SurfPtr;
      
      /* some elements of the structure are not filled at the moment */
      
    } /* end of loop over phi         */
    r_i_old=r_i_new; /* end of one ring */
  }     /* end of loop over rings (n_r) */

  
  /* ---------------------- Calculate inner side of disk --------------------  */
  
  /* -----------------  loop over rings (n_r)  ----------------------   */
  for (i = 0; i < BinPtr->RingsIn ; ++i) {
    phi        = 0.0;   /*  Correct ?????*/
    /* check if enough memory                                             */
    if ((NumElem+n_phi) >= (int) MaximumElements) {
#ifdef _WITH_GTK
      if (Flags.interactive == OFF)
	nf_error(_(errmsg[10]));
      else {
	make_dialog(_(errmsg[10]));
	return(8);
      }
#else
      nf_error(_(errmsg[10]));
#endif
    }
    
      /* ----------------- loop over phi -------------------------------- */
    for (j = 0; j < n_phi; ++j) {
      phi=((2.0*PI/n_phi)*j)-Phase;  /* MPR 23.05.2002 */
      offset_in=-(Thick_in/BinPtr->RingsIn)/2.0;
      
      SurfPtr->lambda      = (float) r_i_old*cos(phi);
      SurfPtr->mu          = (float) r_i_old*sin(phi);
      SurfPtr->nu          = (float) -1.0 *((BinPtr->Thick/2.0+(BinPtr->HR*r_j)
					     /2.0+(offset_in-(i*Thick_in/BinPtr->RingsIn))));
      
      SurfPtr->l           = (float) -1.0 * (cos(phi)); // PR -cos -> cos 21.05.02
      if (fabs(SurfPtr->l) <= DBL_EPSILON) {
	SurfPtr->l = 0.0; }
      SurfPtr->m           = (float) -1.0 * (sin(phi));
      if (fabs(SurfPtr->m) <= DBL_EPSILON) {
	SurfPtr->m = 0.0;}
      SurfPtr->n           = 0;



      TileDisk1(TiltAngle,SurfPtr);	        
      TileDisk2(TiltAngle,SurfPtr);	  

      /* Calculate the layer vectors */
      CalculateLayerVecE1(SurfPtr);
      CalculateLayerVecE2(SurfPtr);

      SurfPtr->AreaType = IN_RECTANGLE;
      SurfPtr->AreaAngle = j*360/n_phi; /* Angle between x-Axis and Middle of the Element */
      SurfPtr->AreaHeight=Thick_in/BinPtr->RingsIn;
      SurfPtr->AreaWidth =(2*PI*Rin)/n_phi;
      SurfPtr->MyNum    = NumElem;
      SurfPtr->rad      = r_j;        /* Is needed for eclipse test */

      /* area is outer side divided by rings/segments:*/
      DSurf            = (2.0*PI*Rin*Thick_out)/n_phi/BinPtr->RingsIn;
      SurfPtr->area    = DSurf;
      Surf             = Surf + DSurf;
      /*  ---------------    INITIALIZE SURFACE TEMP  ----------------- */
      SurfPtr->temp = T1;

      ++NumElem;                     /* increment surface element count */
      ++SurfPtr;
      
      /* some elements of the structure are not filled at the moment */
      
    }    /* end of loop over phi */
  }      /* end of loop over rings (n_r) */
  
  BinPtr->NumElem = NumElem; /* Number of elements*/

  if (Flags.debug[VERBOSE] == ON) { 
    printf("\n"); 
    printf(_(" Divide Disk %3d : %6ld Surface elements\n"),
	   Comp+1, BinPtr->NumElem);
  }

  BinPtr->Surface = Surf;                    /*Added 29-04-2002, EG/MPR */
  //  BinPtr->Gravity       = MeanGrav/Surf;   
  //  BinPtr->DarkeningGrav = MeanDarkGrav/Surf;
  
  /**********************************************************************/
  /*                                                                    */
  /*               Temperature Distribution                             */
  /*                                                                    */
  /**********************************************************************/

  /* Wee need some useful temperature distribution here,                */
  /* e.g. multicolor black body MK                                      */
  /*                                                                    */
  /* --------------- Gravitational Darkening  for disk ???? ----------- */

  /* T propto g**GravDarkCoeff                                          */

  /* SurfPtr = Surface[Comp];

  for (i = 0; i < BinPtr->NumElem; ++i) {

   SurfPtr->temp = 
         BinPtr->Temperature 
       * (exp(BinPtr->GravDarkCoeff * log(SurfPtr->grav)) 
         / BinPtr->DarkeningGrav );

   ++SurfPtr;
 
   } */

  /* ---------------------------- mean temperature  -------------------- */
  
  Weight = 0.0; TempMean = 0.0;
  SurfPtr      = Surface[Comp];
  for (i = 0; i < BinPtr->NumElem; ++i) {
    Area     = SurfPtr->area;
    TempMean = TempMean + SurfPtr->temp * Area;
    Weight   = Weight + Area;
    /*------- Sollte hier nicht irgendwo ein SurfPtr++ stehen?????*/
    /* eigentlich schon ... RW Tue Sep 11 12:54:20 CEST 2001       */
    ++SurfPtr;
  }

  BinPtr->TempMean = TempMean/Weight;
  
  /**********************************************************************/
  /*                                                                    */
  /*               Flux intensity computation                           */
  /*                                                                    */
  /**********************************************************************/

  /* As a first aproximation we use black body emission for the disk    */
  BlackbodyFlux(Disk);

  /* ----------------           output            --------------------- */

  if (Flags.debug[VERBOSE] == ON) { 
    /* printf("\n Star: %5d\n", (Comp+1)); */
    printf(_(" Surface Area, Mean Gravity (dimensionless):  %8.6f %8.6f\n"),
	   BinPtr->Surface, BinPtr->Gravity);
    printf(_(" Temperature is %8.6f\n"), BinPtr->Temperature);
    
    printf("\n");
    printf(_(" Temperature Distribution corrected for: \n")); 
    /*printf(_("    Reflection and Gravity Darkening\n"));
      printf(_("    Coefficient for Gravity Darkening was: %6.4f\n"),
      BinPtr->GravDarkCoeff);*/
  }

  return(0);
}
#endif




